/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/


#include "ContextView/contextscene.h"
#include "EnginePlayer/engineplayer.h"
#include "MediaItem/covercache.h"

/*   info providers   */
#include "provider_lyrics.h"
#include "provider_lastfm.h"
#include "provider_echonest.h"
#include "provider_discogs.h"

#include "utilities.h"
#include "debug.h"

#include <QGraphicsView>
/*
********************************************************************************
*                                                                              *
*    Class ContextScene                                                        *
*                                                                              *
********************************************************************************
*/
ContextScene::ContextScene(QWidget* parent)
{
    m_parentWidget = parent;
    this->setParent(parent);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);

    //! set engine player for update
    m_engine_player = EnginePlayer::instance();
    connect(m_engine_player, SIGNAL(playingInfoChanged()), this, SLOT(slot_update_scene()));
    connect(m_engine_player, SIGNAL(engineStateChanged()), this, SLOT(slot_update_scene()));

    //! init contexte scene
    m_activated     = false;
    m_initialized   = false;

    m_current_track = MEDIA::TrackPtr(0);
    m_request_id    = 0;
    is_new_artist   = false;
    is_new_album    = false;
    is_new_track    = false;
}

ContextScene::~ContextScene()
{
    clear();
}
/*******************************************************************************
  ContextScene::startContextScene
    -> called from graphicsview widget (contextView action on menu triggered)
*******************************************************************************/
void ContextScene::startContextScene()
{
    setActivate(true);

    if( !m_initialized )
      init_scene();

    slot_update_scene();
}

/*******************************************************************************
  ContextScene::init_scene
*******************************************************************************/
void ContextScene::init_scene()
{
    //Debug::debug() << "ContextScene::init_scene ";

    //!  init providers
    m_map_providers.insert("discogs",  new DiscogsProvider() );
    m_map_providers.insert("echonest", new EchonestProvider() );
    m_map_providers.insert("lastfm",   new LastFmProvider() );
    m_map_providers.insert("lyrics",   new LyricsProvider() );

    //Debug::debug() << "ContextScene::init providers";
    foreach (InfoProvider* provider, m_map_providers.values() ) {
      //Debug::debug() << "ContextScene::init provider : " << provider->name();
      provider->init();
    }

    //!  Playing scene widgets
    m_artist_info_widget  = new ArtistInfoWidget(m_parentWidget, static_cast<EchonestProvider*>(m_map_providers.value("echonest")));
    m_similar_info_widget = new ArtistSimilarWidget(m_parentWidget, static_cast<EchonestProvider*>(m_map_providers.value("echonest")));
    m_disco_info_widget   = new DiscoInfoWidget(m_parentWidget, static_cast<DiscogsProvider*>(m_map_providers.value("discogs")));
    m_album_info_widget   = new AlbumInfoWidget(m_parentWidget, static_cast<LastFmProvider*>(m_map_providers.value("lastfm")));
    m_lyrics_info_widget  = new LyricsInfoWidget(m_parentWidget,static_cast<LyricsProvider*>( m_map_providers.value("lyrics") ) );

    connect(m_artist_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    connect(m_similar_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    connect(m_disco_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    connect(m_album_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));
    connect(m_lyrics_info_widget, SIGNAL(updated()), this, SLOT(slot_update_draw()));

    m_artist_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    m_similar_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    m_disco_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    m_album_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    m_lyrics_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );

    
    m_headeritem = new Header();
    m_headeritem->setText(tr("Context view"));
    
    //! now playing widget
    m_nowplaying_info_widget = new NowPlayingInfoWidget(m_parentWidget);
    connect(m_engine_player, SIGNAL(streamInfoChanged()), m_nowplaying_info_widget, SLOT(update()));
    m_nowplaying_info_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );

    //! layout 
    m_layout    = new QGraphicsLinearLayout(Qt::Vertical);
    m_layout->setSpacing(5);
    m_layout->addItem(m_headeritem);
    m_layout->addItem(m_nowplaying_info_widget);    
    m_layout->addItem(m_artist_info_widget);
    m_layout->addItem(m_similar_info_widget);
    m_layout->addItem(m_disco_info_widget);
    m_layout->addItem(m_album_info_widget);
    m_layout->addItem(m_lyrics_info_widget);

    m_container = new QGraphicsWidget();
    m_container->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    m_container->setLayout(m_layout);
    m_container->setPos(0,5);

    this->addItem(m_container);

    m_initialized = true;
    
    clear_scene(false);
}



/*******************************************************************************
  clear_scene
*******************************************************************************/
void ContextScene::clear_scene(bool isplaying)
{
    Debug::debug() << "ContextScene::clear_scene ";
    
    if(isplaying) {
      m_nowplaying_info_widget->hide();
      m_artist_info_widget->show();
      m_similar_info_widget->show();
      m_disco_info_widget->show();
      m_album_info_widget->show();
      m_lyrics_info_widget->show();
    }
    else
    {
      m_artist_info_widget->hide();
      m_similar_info_widget->hide();
      m_disco_info_widget->hide();
      m_album_info_widget->hide();
      m_lyrics_info_widget->hide();
      m_nowplaying_info_widget->show();
    }
    
    m_layout->invalidate();
    m_headeritem->_width  = m_parentWidget->width()-10;
      
    setSceneRect ( itemsBoundingRect());
}

/*******************************************************************************
  ContextScene::analyse_new_media
*******************************************************************************/
bool ContextScene::analyse_new_media()
{
    //Debug::debug() << "ContextScene::analyse_new_media " << m_engine_player->playingUrl();

    if(m_url == m_engine_player->playingUrl())
      return false;  // NO need to update the scene
    else
      m_url = m_engine_player->playingUrl();

    is_new_artist = true;
    is_new_album  = true;
    is_new_track  = true;

    // get new change
    MEDIA::TrackPtr player_media = m_engine_player->playingTrack();

    if(m_current_track)
    {
      if(m_current_track->type() != player_media->type())
      {
        m_current_track.reset();
        m_current_track =  MEDIA::TrackPtr(player_media);
        return true;  // need to update the scene
      }
      else
      {
        is_new_artist = (m_current_track->artist == player_media->artist ) ? false : true;

        is_new_album = (m_current_track->album == player_media->album ) ? false : true;

        is_new_track = (m_current_track->title == player_media->title ) ? false : true;

        m_current_track.reset();
        m_current_track = MEDIA::TrackPtr(0);
      }
    }

    m_current_track =  MEDIA::TrackPtr(player_media);

    return true;
}




/*******************************************************************************
  ContextScene::slot_update_scene
*******************************************************************************/
void ContextScene::slot_update_scene()
{
    if(!isActivated()) return;
    Debug::debug() << "ContextScene::slot_update_scene ";

    /*-------------------------------------------------*/
    /*  PLAYING state                                  */
    /* ------------------------------------------------*/
    if(m_engine_player->engineState() != ENGINE::STOPPED && !m_engine_player->playingUrl().isEmpty())
    {
      if( !analyse_new_media() )
        return;

      /*  type STREAM            */
      if(m_current_track->type() == TYPE_STREAM)
      {
        clear_scene(false);
        return;
      }

      /*  type TRACK            */
      if(!m_current_track) {
        Debug::error() << "ContextScene::error updating mediaitem";
        return;
      }

      clear_scene(true);
      //! initialisation des info
      m_request_id++;

      foreach (InfoProvider* provider, m_map_providers.values()) {
        provider->cancel_searches();
        provider->set_media(m_current_track);
      }

      if(is_new_artist) {
        Debug::warning() << "ContextScene -> new artist";
        m_artist_info_widget->clear();
        m_similar_info_widget->clear();
        m_disco_info_widget->clear();

        m_artist_info_widget->set_artist_name( m_current_track->artist );

        m_artist_info_widget->set_request_id(m_request_id);
        m_similar_info_widget->set_request_id(m_request_id);
        m_disco_info_widget->set_request_id(m_request_id);

        m_map_providers.value("echonest")->fetch_info(m_request_id);
        m_map_providers.value("discogs")->fetch_info(m_request_id);
      }

      if(is_new_album) {
        Debug::warning() << "ContextScene -> new album";
        m_album_info_widget->clear();

        m_album_info_widget->set_album_name( m_current_track->album );
        m_album_info_widget->set_album_pix( CoverCache::instance()->cover(m_current_track) );

        m_album_info_widget->set_request_id(m_request_id);
        m_map_providers.value("lastfm")->fetch_info(m_request_id);
      }

      if(is_new_track) {
        Debug::warning() << "ContextScene -> new track";
        m_lyrics_info_widget->clear();
        m_lyrics_info_widget->set_song_name( m_current_track->title );
        m_lyrics_info_widget->set_request_id(m_request_id);
        m_map_providers.value("lyrics")->fetch_info(m_request_id);
      }

      slot_update_draw();
    }
    /*-------------------------------------------------*/
    /*  STOPPED state                                  */
    /* ------------------------------------------------*/
    //! with VLC add (m_engine_player->playingUrl().isEmpty()) CONDITION
    else if(m_engine_player->engineState() == ENGINE::STOPPED && m_engine_player->playingUrl().isEmpty())
    {
      //Debug::warning() << "ContextScene -> ENGINE::STOPPED";
      clear_scene(false);
      m_url.clear();

      if(m_current_track) {
        m_current_track.reset();
        m_current_track = MEDIA::TrackPtr(0);
      }

      setSceneRect ( QRect(0,0,m_parentWidget->width(), m_parentWidget->height()));
      return;
    }
}


/*******************************************************************************
  ContextScene::slot_update_draw
*******************************************************************************/
void ContextScene::slot_update_draw()
{
    //Debug::debug() << "ContextScene::slot_update_draw ";
    m_layout->invalidate();
    m_container->update();

    QGraphicsView* view = qobject_cast<QGraphicsView*> (m_parentWidget);
    int offset = view->verticalScrollBar()->isVisible() ? view->verticalScrollBar()->width() : 0;
    m_headeritem->_width  = view->width()-10-offset;
    
      
    int diff = m_container->size().height() - itemsBoundingRect().height() + 40;
    int ajust  = ( diff > 0) ? diff : 0;
    setSceneRect ( itemsBoundingRect().adjusted(0, 0, 0, ajust) );
}
