/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#include "context_widget.h"

#include "lyricseditor.h"
#include "EnginePlayer/engineplayer.h"
#include "utilities.h"      // CONFIGDIR
#include "debug.h"

/*   info providers   */
#include "provider_lyrics.h"
#include "provider_lastfm.h"
#include "provider_echonest.h"
#include "provider_discogs.h"

/*
********************************************************************************
*                                                                              *
*    Class ArtistInfoWidget                                                    *
*                                                                              *
********************************************************************************
*/
ArtistInfoWidget::ArtistInfoWidget(QWidget* parentView, EchonestProvider* provider) :
  QGraphicsWidget(0)
{
    m_parent       = parentView;
    m_provider     = provider;
    m_request_id   = -1;

    m_title   = new TitleGraphicItem();

    m_bio     =  new TextGraphicItem();
    m_bio->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Expanding );

    m_header = new QLabel();
    m_header->setAttribute( Qt::WA_NoSystemBackground, true );
    m_header->setTextFormat(Qt::RichText);

    m_button  = new WebLinkItem();
    m_button->setPixmap(QPixmap(":/images/weblink-18x18.png"));
    m_button->setText(tr("LastFm Link"));

    m_image = new QLabel;
    m_image->setAttribute( Qt::WA_NoSystemBackground, true );
    m_image->setFixedSize( 128, 128 );

    QGraphicsProxyWidget *header_proxy_widget = new QGraphicsProxyWidget( this );
    header_proxy_widget->setWidget( m_header );
    header_proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );

    QGraphicsProxyWidget *image_proxy_widget = new QGraphicsProxyWidget( this );
    image_proxy_widget->setWidget( m_image );
    image_proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );



    QGraphicsLinearLayout* m_layout_v = new QGraphicsLinearLayout( Qt::Vertical );
    m_layout_v->addItem( header_proxy_widget );
    m_layout_v->addItem( m_bio );
    m_layout_v->addItem( m_button );

    m_layout_h = new QGraphicsLinearLayout( Qt::Horizontal);
    m_layout_h->setSpacing(10);
    m_layout_h->addItem( image_proxy_widget );
    m_layout_h->addItem( m_layout_v );

    m_layout   = new QGraphicsLinearLayout( Qt::Vertical , this);
    m_layout->addItem( m_title );
    m_layout->addItem( m_layout_h );

    connect(m_provider, SIGNAL(ArtistBioOK(int, QString, QString)), this, SLOT(slot_artist_bio_ready(int,QString, QString)));
    connect(m_provider, SIGNAL(ArtistImageOK(int, QByteArray)), this, SLOT(slot_artist_image_ready(int,QByteArray)));
    connect(m_bio, SIGNAL(linkActivated(const QString &)), this, SLOT(slot_link_activated(const QString&)));

}

void ArtistInfoWidget::update()
{
    prepareGeometryChange();

    m_layout_h->invalidate();
    m_layout->invalidate();

    updateGeometry();

    QGraphicsWidget::update();

    emit updated();
}

QSizeF ArtistInfoWidget::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    int h1 = 50; // title height

    return QSizeF( m_parent->width(),
                   qMax(h1 + m_image->height(), h1 + m_bio->height() + m_button->height() + 20)
                 );
}

void ArtistInfoWidget::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    m_title->m_width =  m_parent->width();
    m_bio->m_width   =  m_parent->width() - m_image->width() -30;

    this->update();
}

void ArtistInfoWidget::set_artist_name(const QString &name)
{
    m_title->m_name  = name;
    m_button->setLink("http://www.last.fm/music/" + name + "/+wiki");
}

void ArtistInfoWidget::clear()
{
    m_image->clear();
    m_bio->clear();
    m_header->setText(QString("<html><b>%1</b></html>").arg(tr("Biography")));
    m_full_plain_bio = QString ( tr("No information found") );

    set_short_bio();
}

void ArtistInfoWidget::slot_link_activated(const QString& link)
{
    //Debug::debug() << " #### link activated " << link;
    if(link == "more")
      set_long_bio();
    else
      set_short_bio();

    m_bio->updateItem();
    this->update();
}

//! ------------------ slot_artist_bio_ready -----------------------------------
void ArtistInfoWidget::slot_artist_bio_ready(int id, QString provider, QString bio)
{
    //Debug::debug() << "ArtistInfoWidget::slot_artist_bio_ready m_request_id " << m_request_id;
    //Debug::debug() << "ArtistInfoWidget::slot_artist_bio_ready id " << id;
    if(m_request_id != id) return;
    m_full_plain_bio = bio;

    m_header->setText(QString("<html><b>%1</b>  (%2)</html>").arg(tr("Biography"),provider));

    set_short_bio();
    m_bio->updateItem();

    this->update();
}

void ArtistInfoWidget::set_short_bio()
{
    int split  = m_full_plain_bio.indexOf('\n', 1024);
    if(split == -1)
      split = m_full_plain_bio.indexOf(". ", 1024);

    QString html_text;
    html_text += QString("<p align=\"justify\">");
    html_text += m_full_plain_bio.left(split);

    if (split != -1) {
      html_text += "...";
      html_text += QString("<a href='more'>more</a>");
    }
    html_text += "</p>";

    m_bio->setHtml(html_text);
}

void ArtistInfoWidget::set_long_bio()
{
    QString html_text;
    html_text += QString("<p align=\"justify\">");
    html_text += m_full_plain_bio;

    html_text += QString("<a href='less'>less</a>");
    html_text += "</p>";

    m_bio->setHtml(html_text);
}

//! ------------------ slot_artist_image_ready ---------------------------------
void ArtistInfoWidget::slot_artist_image_ready(int id, QByteArray bytes)
{
    if(m_request_id != id) return;

    //! update artist image
    QImage image = QImage::fromData(bytes);
    if(image.isNull()) return;

    QPixmap pix = QPixmap::fromImage(image);
    pix = pix.scaledToWidth (128, Qt::SmoothTransformation);

    m_image->setPixmap( pix );

    this->update();
}

/*
********************************************************************************
*                                                                              *
*    Class ArtistSimilarWidget                                                 *
*                                                                              *
********************************************************************************
*/
ArtistSimilarWidget::ArtistSimilarWidget(QWidget* parentView, EchonestProvider* provider) :
  QGraphicsWidget(0)
{
    m_parent       = parentView;
    m_provider     = provider;
    m_request_id   = -1;

    m_header = new QLabel();
    m_header->setAttribute( Qt::WA_NoSystemBackground, true );
    m_header->setTextFormat(Qt::RichText);
    m_header->setText(QString("<html><b>%1</b></html>").arg(tr("Similars artists")));

    QGraphicsProxyWidget *proxy = new QGraphicsProxyWidget( this );
    proxy->setWidget( m_header );
    proxy->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    proxy->setPos(20,0);


    connect(m_provider, SIGNAL(ArtistSimilarOK(int, QStringList)), this, SLOT(slot_artist_similar_ready(int,QStringList)));
}

void ArtistSimilarWidget::clear()
{
    qDeleteAll(m_tags);
    m_tags.clear();
}

void ArtistSimilarWidget::update()
{
    prepareGeometryChange();

    updateGeometry();

    doLayout(true);

    QGraphicsWidget::update();

    emit updated();
}

QSizeF ArtistSimilarWidget::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    return doLayout(false);
}

void ArtistSimilarWidget::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    this->update();
}

//! ------------------ slot_artist_similar_ready -------------------------------
void ArtistSimilarWidget::slot_artist_similar_ready(int id,QStringList list)
{
    //Debug::debug() << "ArtistSimilarWidget::slot_artist_similar_ready id " << id;

    if(m_request_id != id) return;

    foreach (QString artist , list)
    {
      //Debug::debug() << "ArtistSimilarWidget:: similar : " << artist;

      TagGraphicItem* tag = new TagGraphicItem();
      tag->m_name = artist;
      tag->updateItem();
      m_tags.append(tag);
      tag->setParentItem(this);
    }
    this->update();
}


QSizeF ArtistSimilarWidget::doLayout(bool redraw) const
{
     //Debug::debug() << "ArtistSimilarWidget::doLayout ";
     qreal left, top, right, bottom;
     getContentsMargins(&left, &top, &right, &bottom);

     const int HSpacing = 6;
     const int VSpacing = 6;
     left   += 20;
     right  += 20;
     top    += 30;
     bottom += 10;

     const qreal maxw = m_parent->width() - left - right;

     qreal x = 0;
     qreal y = 0;
     qreal maxRowHeight = 0;
     QSizeF pref;
     for (int i = 0; i < m_tags.count(); ++i) {
         TagGraphicItem* tag = m_tags.at(i);
         pref = tag->effectiveSizeHint(Qt::PreferredSize);
         maxRowHeight = qMax(maxRowHeight, pref.height());

         qreal next_x;
         next_x = x + pref.width();
         if (next_x > maxw) {
             if (x == 0) {
                 pref.setWidth(maxw);
             } else {
                 x = 0;
                 next_x = pref.width();
             }
             y += maxRowHeight + VSpacing;
             maxRowHeight = 0;
         }
         if(redraw)
           tag->setPos(QPointF(left + x, top + y));

         x = next_x + HSpacing;
     }
     maxRowHeight = qMax(maxRowHeight, pref.height());

     return QSizeF(maxw, top + y + maxRowHeight + bottom);
}



/*
********************************************************************************
*                                                                              *
*    Class DiscoInfoWidget                                                     *
*                                                                              *
********************************************************************************
*/
DiscoInfoWidget::DiscoInfoWidget(QWidget* parentView, DiscogsProvider* provider) :
  QGraphicsWidget(0)
{
    m_parent       = parentView;
    m_provider     = provider;
    m_request_id   = -1;

    m_header = new QLabel();
    m_header->setAttribute( Qt::WA_NoSystemBackground, true );
    m_header->setTextFormat(Qt::RichText);
    m_header->setText(QString("<html><b>%1</b></html>").arg(tr("Discography")));

    QGraphicsProxyWidget *proxy = new QGraphicsProxyWidget( this );
    proxy->setWidget( m_header );
    proxy->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    proxy->setPos(20,0);

    m_button  = new WebLinkItem();
    m_button->setParentItem(this);

    connect(m_provider, SIGNAL(artist_data_received(int,const DISCOGS::Artist&)), this, SLOT(slot_artist_data_received(int,const DISCOGS::Artist&)));
    connect(m_provider, SIGNAL(release_cover_received(int,int,QByteArray)), this, SLOT(slot_cover_received(int,int,QByteArray)));
}

void DiscoInfoWidget::clear()
{
    Debug::debug() << "DiscoInfoWidget::clear ";
    qDeleteAll(m_map_albums);
    m_map_albums.clear();

    m_artist.m_releases.clear();

    m_button->clear();
    m_button->setPixmap(QPixmap(":/images/weblink-18x18.png"));
    m_button->setText(tr("Discogs Link"));
}

void DiscoInfoWidget::update()
{
    prepareGeometryChange();

    updateGeometry();

    doLayout(true);

    QGraphicsWidget::update();

    emit updated();
}

QSizeF DiscoInfoWidget::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    return doLayout(false);
}

void DiscoInfoWidget::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    this->update();
}

QSizeF DiscoInfoWidget::doLayout(bool redraw) const
{
     //Debug::debug() << "DiscoInfoWidget::doLayout ";
     qreal left, top, right, bottom;
     getContentsMargins(&left, &top, &right, &bottom);

     const int HSpacing = 6;
     const int VSpacing = 6;
     left   += 20;
     right  += 20;
     top    += 30;
     bottom += 30;

     const qreal maxw = m_parent->width() - left - right;

     qreal x = 0;
     qreal y = 0;
     qreal maxRowHeight = 0;
     QSizeF pref;

     QList<AlbumThumbGraphicItem*> albums = m_map_albums.values();

     for (int i = 0; i < albums.count(); ++i) {
         AlbumThumbGraphicItem* album = albums.at(i);
         pref = album->effectiveSizeHint(Qt::PreferredSize);
         maxRowHeight = qMax(maxRowHeight, pref.height());

         qreal next_x;
         next_x = x + pref.width();
         if (next_x > maxw) {
             if (x == 0) {
                 pref.setWidth(maxw);
             } else {
                 x = 0;
                 next_x = pref.width();
             }
             y += maxRowHeight + VSpacing;
             maxRowHeight = 0;
         }
         if(redraw) {
           album->setPos(QPointF(left + x, top + y));
           //Debug::warning() << "DiscoInfoWidget -> album setPos x :  " << left + x << "y:" << top + y ;
         }

         x = next_x + HSpacing;
     }
     maxRowHeight = qMax(maxRowHeight, pref.height());

     m_button->setPos(20, top + y + maxRowHeight + HSpacing);
     return QSizeF(maxw, top + y + maxRowHeight + bottom);
}

void DiscoInfoWidget::slot_artist_data_received(int id,const DISCOGS::Artist& artist)
{
    Debug::debug() << "DiscoInfoWidget::slot_artist_data_received";

    if(m_request_id != id) return;

    /* deep structure copy */
    m_artist = artist;

    foreach(DISCOGS::Release release, m_artist.m_releases) {
        if(m_map_albums.keys().contains(release.m_id)) {
          Debug::warning() << "DiscoInfoWidget -> release already found id" << release.m_id;
          continue;
        }
        //Debug::debug() << "DiscoInfoWidget -> release found OK !!! ";

        AlbumThumbGraphicItem* album_item = new AlbumThumbGraphicItem();

        album_item->m_title = release.m_title;
        album_item->m_year  = release.m_year;
        album_item->setParentItem(this);
        m_map_albums[release.m_id] = album_item;
    }

    m_button->setLink(artist.m_discogs_url);

    this->update();
}

void DiscoInfoWidget::slot_cover_received(int id,int release_id, QByteArray bytes)
{
    //Debug::debug() << "DiscoInfoWidget::slot_cover_received  m_request_id " << m_request_id;
    //Debug::debug() << "DiscoInfoWidget::slot_cover_received  release_id " << release_id;

    if(m_request_id != id) return;

    for(int i=0; i< m_artist.m_releases.size();i++) {
      if(m_artist.m_releases.at(i).m_id == release_id)
      {
          if(m_map_albums.contains(release_id))
          {
            QImage image = QImage::fromData(bytes);
            if( !image.isNull() ) {
              image = image.scaled(QSize(64, 64), Qt::KeepAspectRatio, Qt::FastTransformation);

              m_map_albums[release_id]->m_pix = QPixmap::fromImage(image);
              m_map_albums[release_id]->update();
            }
          }
          break;
      }
    }
}



/*
********************************************************************************
*                                                                              *
*    Class AlbumInfoWidget                                                     *
*                                                                              *
********************************************************************************
*/
AlbumInfoWidget::AlbumInfoWidget(QWidget* parentView, LastFmProvider* provider) :
  QGraphicsWidget(0)
{
    m_parent       = parentView;
    m_provider     = provider;
    m_request_id   = -1;

    m_title   =  new TitleGraphicItem();
    m_bio     =  new TextGraphicItem();
    m_bio->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Expanding );

    m_header = new QLabel();
    m_header->setAttribute( Qt::WA_NoSystemBackground, true );
    m_header->setTextFormat(Qt::RichText);

    m_image   = new QLabel;
    m_image->setAttribute( Qt::WA_NoSystemBackground, true );
    m_image->setFixedSize( 110, 110 );

    QGraphicsProxyWidget *header_proxy_widget = new QGraphicsProxyWidget( this );
    header_proxy_widget->setWidget( m_header );
    header_proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );

    QGraphicsProxyWidget *image_proxy_widget = new QGraphicsProxyWidget( this );
    image_proxy_widget->setWidget( m_image );
    image_proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );


    QGraphicsLinearLayout* m_layout_v = new QGraphicsLinearLayout( Qt::Vertical );
    m_layout_v->addItem( header_proxy_widget );
    m_layout_v->addItem( m_bio );

    m_layout_h = new QGraphicsLinearLayout( Qt::Horizontal );
    m_layout_h->setSpacing(28);
    m_layout_h->addItem( image_proxy_widget );
    m_layout_h->addItem( m_layout_v );

    m_layout   = new QGraphicsLinearLayout( Qt::Vertical , this);
    m_layout->addItem( m_title );
    m_layout->addItem( m_layout_h );


    connect(m_provider, SIGNAL(AlbumBioOK(int, QString)), this, SLOT(slot_album_bio_ready(int,QString)));
    connect(m_provider, SIGNAL(AlbumImageOK(int, QByteArray)), this, SLOT(slot_album_image_ready(int,QByteArray)));
}

void AlbumInfoWidget::clear()
{
    m_image->clear();
    m_header->setText(QString("<html><b>%1</b>  (%2)</html>").arg(tr("Album information"),"lastfm"));
    m_bio->clear();
    m_bio->setHtml( QString(tr("No information found")) );

}

void AlbumInfoWidget::update()
{
    prepareGeometryChange();

    updateGeometry();

    m_layout_h->invalidate();
    m_layout->invalidate();

    QGraphicsWidget::update();

    emit updated();
}

void AlbumInfoWidget::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    m_title->m_width =  m_parent->width();
    m_bio->m_width   =  m_parent->width() - m_image->width() -30;

    this->update();
}

void AlbumInfoWidget::set_album_name(const QString &name)
{
    m_title->m_name = name;
    this->update();
}


void AlbumInfoWidget::set_album_pix(const QPixmap &p)
{
    m_image->setPixmap( p );
    m_image->update();
}


QSizeF AlbumInfoWidget::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    int h1 = 50; // title height
    return QSizeF( m_parent->width(), qMax(h1 + 110 /*image*/, h1 + m_bio->height()) );
}
/*******************************************************************************
  AlbumInfoWidget::slot_album_bio_ready
*******************************************************************************/
void AlbumInfoWidget::slot_album_bio_ready(int id, QString bio)
{
    //Debug::debug() << " ---- ContextScene::slot_album_bio_ready";
    if(m_request_id != id) return;

    int split = bio.indexOf('\n', 1024);
    if (split == -1) {
      split = bio.indexOf(". ", 1024);
    }


    QString html_text = QString("<p align=\"justify\">");
    html_text += bio.left(split);
    html_text += "</p>";

    //! update album bio
    m_bio->setHtml(html_text);
    m_bio->updateItem();
    this->update();
}

/*******************************************************************************
  AlbumInfoWidget::slot_album_image_ready
*******************************************************************************/
void AlbumInfoWidget::slot_album_image_ready(int id, QByteArray bytes)
{
    if(m_request_id != id) return;

    if(!m_image->pixmap()->isNull()) return ;

    //! update artist image
    QImage image = QImage::fromData(bytes);
    if(image.isNull()) return;

    QPixmap pix = QPixmap::fromImage(image);
    pix = pix.scaledToWidth (110, Qt::SmoothTransformation);

    m_image->setPixmap( pix );
}




/*
********************************************************************************
*                                                                              *
*    Class LyricsInfoWidget                                                    *
*                                                                              *
********************************************************************************
*/
LyricsInfoWidget::LyricsInfoWidget(QWidget* parentView, LyricsProvider* d) :
  QGraphicsWidget(0)
{
    //Debug::debug() << " LyricsInfoWidget constructor";

    m_parent       = parentView;
    m_provider     = d;
    m_request_id   = -1;
    lyrics_found   = false;

    m_title         = new TitleGraphicItem();
    m_lyrics        = new TextGraphicItem();
    m_lyrics->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Expanding );

    m_header = new QLabel();
    m_header->setAttribute( Qt::WA_NoSystemBackground, true );
    m_header->setTextFormat(Qt::RichText);

    QGraphicsProxyWidget *header_proxy_widget = new QGraphicsProxyWidget( this );
    header_proxy_widget->setWidget( m_header );
    header_proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );


    m_button_add    = new ButtonItem();
    m_button_add->setText(tr("Add file"));
    m_button_add->setPixmap(QPixmap(":/images/document-new-48x48.png"));
    connect(m_button_add, SIGNAL(clicked()), this, SLOT(slot_add_lyrics()));

    m_button_remove = new ButtonItem();
    m_button_remove->setPixmap(QPixmap(":/images/document-close-32x32.png"));
    m_button_remove->setText(tr("Remove file"));
    m_button_remove->hide();
    connect(m_button_remove, SIGNAL(clicked()), this, SLOT(slot_remove_lyrics()));

    m_button_link  = new WebLinkItem();
    m_button_link->setPixmap(QPixmap(":/images/weblink-18x18.png"));
    m_button_link->setText("Lyrics Link");
    m_button_link->hide();

    QGraphicsLinearLayout* m_layout_v = new QGraphicsLinearLayout( Qt::Vertical);
    m_layout_v->addItem( m_button_add );
    m_layout_v->addItem( m_button_remove );
    m_layout_v->addItem( m_button_link );

    QGraphicsLinearLayout* m_layout_v2 = new QGraphicsLinearLayout( Qt::Vertical);
    m_layout_v2->addItem( header_proxy_widget );
    m_layout_v2->addItem( m_lyrics );

    m_layout_h  = new QGraphicsLinearLayout( Qt::Horizontal);
    m_layout_h->addItem( m_layout_v );
    m_layout_h->addItem( m_layout_v2 );
    m_layout_h->setItemSpacing ( 0, 50 );

    m_layout    = new QGraphicsLinearLayout( Qt::Vertical , this);
    m_layout->addItem( m_title );
    m_layout->addItem( m_layout_h );

    connect(m_provider, SIGNAL(LyricsOK(int, LYRICS::Result&)), this, SLOT(slot_lyrics_ready(int, LYRICS::Result&)));
}

void LyricsInfoWidget::clear()
{
    lyrics_found = false;
    m_lyrics->clear();
    m_lyrics->setHtml( QString(tr("No lyrics found")) );

    m_header->setText(QString("<html><b>%1</b></html>").arg(tr("Song lyrics")) );
}

void LyricsInfoWidget::update()
{
    prepareGeometryChange();

    updateGeometry();

    m_layout_h->invalidate();
    m_layout->invalidate();

    QGraphicsWidget::update();

    emit updated();
}

QSizeF LyricsInfoWidget::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    int h1 = 50; // title height
    return QSizeF(m_parent->width(),
       qMax(h1 + m_button_add->height()*2,h1 + m_lyrics->height()));
}

void LyricsInfoWidget::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    m_title->m_width    = m_parent->width();
    m_lyrics->m_width   = m_parent->width() -30;

    this->update();
}

void LyricsInfoWidget::set_song_name(const QString &name)
{
    m_title->m_name = name;
    m_lyrics->updateItem();
}



/*******************************************************************************
  LyricsInfoWidget::slot_lyrics_ready
*******************************************************************************/
void LyricsInfoWidget::slot_lyrics_ready(int id, LYRICS::Result& result)
{
    if(m_request_id != id) return;

    Debug::debug() << " LyricsInfoWidget::slot_lyrics_ready";
    //Debug::debug() << " LyricsInfoWidget::slot_lyrics_ready lyrics " << lyrics;

    if(lyrics_found) return; // lyrics already found

    //! local provider
    if(result.m_provider_name == "local")
    {
       m_button_remove->show();
       m_button_add->setText(tr("Edit file"));
       m_button_add->setPixmap(QPixmap(":/images/edit_info.png"));
    }
    else
    {
      m_button_remove->hide();

      m_button_add->setText(tr("Add file"));
      m_button_add->setPixmap(QPixmap(":/images/document-new-48x48.png"));

      m_button_link->setLink(result.m_url);
      m_button_link->show();
    }

    //! get plain text for lyrics editor
    m_header->setText(QString("<body><html><b>%1</b> (%2)").arg(tr("Song lyrics"), result.m_provider_name) );

    QString htmlLyrics = "<body><html>";
    htmlLyrics += result.m_lyrics;
    htmlLyrics += "</body></html>";

    m_lyrics->setHtml( htmlLyrics );
    m_lyrics->updateItem();

    lyrics_found = true;

    this->update();
}


/*******************************************************************************
  LyricsInfoWidget::slot_add_lyrics
*******************************************************************************/
void LyricsInfoWidget::slot_add_lyrics()
{
    Debug::debug() << " ---- LyricsInfoWidget::slot_add_lyrics";
    //! Show settings dialog
    LyricsEditor *le = new LyricsEditor(m_parent);
    MEDIA::TrackPtr media = EnginePlayer::instance()->playingTrack();

    if(media->type() == TYPE_TRACK)
      le->setInfo( media->artist,media->title,m_request_id );

    if(lyrics_found)
      le->setLyrics( m_lyrics->toPlainText() );

    connect(le,SIGNAL(LyricsChanged(int,LYRICS::Result&)),this, SLOT(slot_lyrics_ready(int,LYRICS::Result&)));
    le->exec();
}

/*******************************************************************************
  LyricsInfoWidget::slot_remove_lyrics
*******************************************************************************/
void LyricsInfoWidget::slot_remove_lyrics()
{
    Debug::debug() << " ---- LyricsInfoWidget::slot_remove_lyrics";

    QCryptographicHash hash(QCryptographicHash::Sha1);

    MEDIA::TrackPtr media = EnginePlayer::instance()->playingTrack();

    if(media->type() == TYPE_TRACK) {
      hash.addData(media->artist.toLower().toUtf8().constData());
      hash.addData(media->title.toLower().toUtf8().constData());

      QString path = QString(UTIL::CONFIGDIR + "/lyrics/" + hash.result().toHex() + ".txt");

      QFile file(path);
      if(file.exists()) {
        file.remove();
        Debug::debug() << " ---- LyricsInfoWidget::slot_remove_lyrics  file removed !";
      }

      lyrics_found = false;
    }
}




/*
********************************************************************************
*                                                                              *
*    Class NowPlayingInfoWidget                                                *
*                                                                              *
********************************************************************************
*/
NowPlayingInfoWidget::NowPlayingInfoWidget(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    m_info    = new NowPlayingGraphicItem();
    m_info->setParentItem(this);
    m_info->setPos(30,10);
}

void NowPlayingInfoWidget::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    update();
}

QSizeF NowPlayingInfoWidget::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
  switch (which) {
     case Qt::MinimumSize:
       return QSizeF( 0, 0);
       break;
     case Qt::PreferredSize:
     case Qt::MaximumSize:
         return QSizeF( m_parent->width()-20, 10 + 180 );
     default:
         break;
     }
     return constraint;
}

void NowPlayingInfoWidget::update()
{
    m_info->update();

    QGraphicsWidget::update();

    emit updated();
}

